//Frank Vloet june 10th 2015
//Final Project Fab Academy, based on code from:
// Neil Gershenfeld
// 10/27/10
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "USI_TWI_Master.h"
#include <avr/pgmspace.h>

#define SLAVE_ADDR  0x10

#define MESSAGEBUF_SIZE       4

#define TWI_GEN_CALL         0x00  // The General Call address is 0

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20

// Sample TWI transmission states, used in the main application.
#define SEND_DATA             0x01
#define REQUEST_DATA          0x02
#define READ_DATA_FROM_BUFFER 0x03

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_out (1 << PA1)

//led pins
#define led_port PORTA
#define led_direction DDRA
#define red (1 << PA3) //red=pa3
#define blue (1 << PA1) //blue=pa1 
#define green (1 << PA2) //green=pa2

#define max_buffer 25

unsigned char TWI_Act_On_Failure_In_Last_Transmission ( unsigned char TWIerrorMsg )
{
	// A failure has occurred, use TWIerrorMsg to determine the nature of the failure
	// and take appropriate actions.
	// Se header file for a list of possible failures messages.
	 asm("nop");
	 return TWIerrorMsg;
}

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   // initialize serial communication
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

void put_string(volatile unsigned char *port, unsigned char pin, char *str) {
   //
   // print a null-terminated string
   //
   static int index;
   index = 0;
   do {
      put_char(port, pin, str[index]);
      ++index;
      } while (str[index] != 0);
   }


int main(void) {
   //
   // main
   //
   OSCCAL = 0x98;
   char chr, valred, valgreen, valblue;
   //
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);
   //
   //
   // intitialise things for I2C
  unsigned char TWI_targetSlaveAddress, temp, myCounter=0xFF;
  unsigned char messageBuf[MESSAGEBUF_SIZE];
   //
   //// master init
  USI_TWI_Master_Initialise();
  
 sei();
  
  TWI_targetSlaveAddress   = 0x10;
   // init A/D
   //
   ADMUX = (0 << REFS1) | (0 << REFS0) // Vcc ref
      | (0 << ADLAR) // right adjust
      | (0 << MUX5) | (0 << MUX4) | (0 << MUX3) | (1 << MUX2) | (1 << MUX1) | (1 << MUX0); // ADC7
   ADCSRA = (1 << ADEN) // enable
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
   //
   //
   // initialize led pins
   //
   set(led_port, red);
   output(led_direction, red);
   set(led_port, blue);
   output(led_direction, blue);
   set(led_port, green);
   output(led_direction, green);
   //
   // main loop
   //
   while (1) {
      //red
      clear(led_port, red);
      set(led_port, blue);
      set(led_port, green);
     _delay_ms(50); //blink red led half a second
      // send framing
      //
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // send result
      //      put_char(&serial_port, serial_pin_out, 'A');
      chr = ADCL;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      chr = ADCH;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      if (ADC  <= 800) {
         valred = 1;
         }
         else {
         valred = 0;
         }
      //
      //green
      set(led_port, red);
      set(led_port, blue);
      clear(led_port, green);
      _delay_ms(50); //blink red led half a second
      // send framing
      //
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // send result
      //      put_char(&serial_port, serial_pin_out, 'A');
      chr = ADCL;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      chr = ADCH;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      if (ADC  <= 900) {
         valgreen = 1;
         }
         else {
         valgreen = 0;
         }
      //
      //blue
      set(led_port, red);
      clear(led_port, blue);
      set(led_port, green);
     _delay_ms(50); //blink red led half a second
      // send framing
      //
      //
      // initiate conversion
      //
      ADCSRA |= (1 << ADSC);
      //
      // wait for completion
      //
      while (ADCSRA & (1 << ADSC))
         ;
      //
      // send result
      //      put_char(&serial_port, serial_pin_out, 'A');
      chr = ADCL;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      chr = ADCH;
      put_char(&serial_port, serial_pin_out, chr);
      char_delay();
      if (ADC  <= 920) {
         valblue = 1;
         }
         else {
         valblue = 0;
         }
      //
      //all leds off
      set(led_port, red);
      set(led_port, blue);
      set(led_port, green);

      //red
      if ((valred  == 1) && (valgreen == 0) && (valblue == 0)) {
         myCounter = 'r';
         }
      //yellow
      if ((valred  == 1) && (valgreen == 1) && (valblue == 0)) {
         myCounter = 'y';
         }
      //green
      if ((valred  == 0) && (valgreen == 1) && (valblue == 0)) {
         myCounter = 'g';
         }
      //cyan
      if ((valred  == 0) && (valgreen == 1) && (valblue == 1)) {
         myCounter = 'c';
         }
      //blue
      if ((valred  == 0) && (valgreen == 0) && (valblue == 1)) {
         myCounter = 'b';
         }
      //magenta
      if ((valred  == 1) && (valgreen == 0) && (valblue == 1)) {
         myCounter = 'm';
         }
      //white
      if ((valred  == 1) && (valgreen == 1) && (valblue == 1)) {
         myCounter = 'w';
         }
      //off
      if ((valred  == 0) && (valgreen == 0) && (valblue == 0)) {
         myCounter = 'o';
         }
	//start communication
  TWI_targetSlaveAddress   = 0x10;
        temp = TRUE;
        // Send a Address Call, sending a command and data to the Slave          
          messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
          messageBuf[1] = TWI_CMD_MASTER_WRITE;             // The first byte is used for commands.
          messageBuf[2] = myCounter;                        // The second byte is used for the data.
          temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );
      if (!temp)    // One of the operations failed.
      {             // Use TWI status information to detemine cause of failure and take appropriate actions. 
        TWI_Act_On_Failure_In_Last_Transmission( USI_TWI_Get_State_Info( ) );  
       asm("nop"); // Put own code here.
     }
      _delay_ms(500); //wait half a second

	//start communication
  TWI_targetSlaveAddress   = 0x20;
        temp = TRUE;
        // Send a Address Call, sending a command and data to the Slave          
          messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
          messageBuf[1] = TWI_CMD_MASTER_WRITE;             // The first byte is used for commands.
          messageBuf[2] = myCounter;                        // The second byte is used for the data.
          temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );

      _delay_ms(500); //wait half a second

	//start communication
  TWI_targetSlaveAddress   = 0x30;
        temp = TRUE;
        // Send a Address Call, sending a command and data to the Slave          
          messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
          messageBuf[1] = TWI_CMD_MASTER_WRITE;             // The first byte is used for commands.
          messageBuf[2] = myCounter;                        // The second byte is used for the data.
          temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );

      _delay_ms(500); //wait half a second

	//start communication
  TWI_targetSlaveAddress   = 0x40;
        temp = TRUE;
        // Send a Address Call, sending a command and data to the Slave          
          messageBuf[0] = (TWI_targetSlaveAddress<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT); // The first byte must always consit of General Call code or the TWI slave address.
          messageBuf[1] = TWI_CMD_MASTER_WRITE;             // The first byte is used for commands.
          messageBuf[2] = myCounter;                        // The second byte is used for the data.
          temp = USI_TWI_Start_Transceiver_With_Data( messageBuf, 3 );

      _delay_ms(500); //wait half a second

      valred = 0; //reset values 
      valgreen = 0;
      valblue = 0;


   }
}
